#include <proto.h>
#ifndef GT_NO_XPM_LIB
#include <X11/xpm.h>
#endif

#include "gtpixmap.h"
#include "gtcolormap.h"
#include "gterror.h"

/************************************************************************\
*
* GtPixmap class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

static GtPixmap *first=NULL;

/************************************************************************\
*
* Create and init pixmap.
*
\************************************************************************/
GtPixmap *GtMkPixmap(GtDisplay *d,GtTag *tgs)
{
	GtPixmap *obj;
	
	if(!(obj=new GtPixmap()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtPixmap object!\n");
		return(NULL);
	}
	if(!obj->Init(d,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init pixmap.
*
\************************************************************************/
GtPixmap *GtMkPixmap(GtDisplay *d,ulong tag,...)
{
	va_list va;
	GtPixmap *obj;
	int p;
	
	va_start(va,tag);
	p=GtBeginVaTags();
	obj=GtMkPixmap(d,GtCopyVaTags(tag,&va));
	GtEndVaTags(p);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Create and init shared pixmap.
*
\************************************************************************/
GtPixmap *GtMkSharedPixmap(GtDisplay *d,GtTag *tgs)
{
	GtTag *t,*tt;
	GtPixmap *obj,*p;
	GtDrawable *pdra=NULL;
	GtDisplay *pdsp=NULL;
	char *pdata=NULL,*pfname=NULL;
	sbool bm=FALSE;
	
	pdsp=d;
	tt=tgs;
	while((t=GtNextTag(&tt)))
	{
		switch(t->tag)
		{
			case GT_Drawable:
				pdra=(GtDrawable *)t->d.ptr;
				break;
			case GT_BitmapData:
				pdata=(char *)t->d.ptr;
				bm=TRUE;
				break;
			case GT_XpmData:
				pdata=(char *)t->d.ptr;
				bm=FALSE;
				break;
			case GT_BitmapName:
				pfname=(char *)t->d.ptr;
				bm=TRUE;
				break;
			case GT_XpmName:
				pfname=(char *)t->d.ptr;
				bm=FALSE;
				break;
		}
	}
	for(p=first; p; p=p->next)
	{
		if((pdsp==p->dsp) && (pdra==p->dra) &&
			(bm && (p->depth==1)) && (pdata==p->data) && (pfname==p->fname))
		{
			p->cnt++;
			return(p);
		}
	}
	if(!(obj=new GtPixmap()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtPixmap object!\n");
		return(NULL);
	}
	obj->next=first;
	first=obj;
	obj->cnt=0;
	if(!obj->Init(d,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}

/************************************************************************\
*
* Create and init shared pixmap.
*
\************************************************************************/
GtPixmap *GtMkSharedPixmap(GtDisplay *d,ulong tag,...)
{
	va_list va;
	GtPixmap *obj;
	int p;
	
	va_start(va,tag);
	p=GtBeginVaTags();
	obj=GtMkSharedPixmap(d,GtCopyVaTags(tag,&va));
	GtEndVaTags(p);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Delete shared pixmap.
*
\************************************************************************/
void GtPixmap::DeInit(void)
{
	GtPixmap *t;
	
	if(cnt>0)
	{
		cnt--;
		return;
	}
	if(cnt==0)
	{
		t=first;
		if(t==this)
			first=next;
		else
		{
			while(t && (t->next!=this))
				t=t->next;
			if(t)
				t->next=next;
		}
	}
	GtDrawable::DeInit();
}

/************************************************************************\
*
* Return TRUE if Xpm support compiled in.
*
\************************************************************************/
sbool GtXpmSupport(void)
{
#ifdef GT_NO_XPM_LIB
	return(FALSE);
#else
	return(TRUE);
#endif
}

/************************************************************************\
*
* Setup empty pixmap.
*
\************************************************************************/
GtPixmap::GtPixmap(void)
{
	dra=NULL;
	pix=0;
	mask=0;
	xsize=ysize=1;
	hotx=hoty=0;
	depth=1;
	data=NULL;
	fname=NULL;
	cnt=-1;
	nofreex=FALSE;
}

/************************************************************************\
*
* Destroy pixmap.
*
\************************************************************************/
void GtPixmap::Destroy(void)
{
	if(pix && !nofreex)
		XFreePixmap(dsp->XDsp(),pix);
	if(mask && !nofreex)
		XFreePixmap(dsp->XDsp(),mask);
	GtDrawable::Destroy();
}

/************************************************************************\
*
* Setup pixmap.
*
\************************************************************************/
sbool GtPixmap::Init(GtDisplay *d,GtTag *tgs)
{
	GtTag *t;
	Drawable draw;
	sbool bm=FALSE,bgset=FALSE;
	GtColormap *cm=NULL;
	ulong fg,bg;
#ifndef GT_NO_XPM_LIB
	XpmAttributes attr;
	XpmColorSymbol csym;
#endif

	if(!GtDrawable::Init(d,tgs))
		return(FALSE);
	fg=dsp->XBlackPixel();
	bg=dsp->XWhitePixel();
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Drawable:
				dra=(GtDrawable *)t->d.ptr;
				break;
			case GT_BitmapData:
				data=(char *)t->d.ptr;
				bm=TRUE;
				break;
			case GT_XpmData:
				data=(char *)t->d.ptr;
				bm=FALSE;
				break;
			case GT_BitmapName:
				fname=(char *)t->d.ptr;
				bm=TRUE;
				break;
			case GT_XpmName:
				fname=(char *)t->d.ptr;
				bm=FALSE;
				break;
			case GT_XSize:
				xsize=t->d.val;
				break;
			case GT_YSize:
				ysize=t->d.val;
				break;
			case GT_Depth:
				depth=t->d.val;
				break;
			case GT_HotX:
				hotx=t->d.val;
				break;
			case GT_HotY:
				hoty=t->d.val;
				break;
			case GT_Colormap:
				cm=(GtColormap *)t->d.ptr;
				break;
			case GT_Foreground:
				fg=t->d.val;
				break;
			case GT_Background:
				bg=t->d.val;
				bgset=TRUE;
				break;
			case GT_XPixmap:
				pix=(Pixmap)t->d.ptr;
				nofreex=TRUE;
				break;
			case GT_DontFreeX:
				if(t->d.val)
					nofreex=TRUE;
				else
					nofreex=FALSE;
				break;
		}
	}
	if(pix)
	{
		dr=pix;
		return(TRUE);
	}
	if(dra)
		draw=dra->XDr();
	else
		draw=dsp->XDefaultRootWindow();
	if(fname)
	{
		if(bm)
		{
			if(XReadBitmapFileData(fname,&xsize,&ysize,(unsigned char **)&data,
													&hotx,&hoty)!=BitmapSuccess)
			{
				GtError(this,GTEC_NOXRESOURCE,"No bitmap file: %s\n",fname);
				return(FALSE);
			}
		}
		else
#ifndef GT_NO_XPM_LIB
		{
			if(XpmReadFileToData(fname,(char ***)&data)!=XpmSuccess)
			{
				GtError(this,GTEC_NOXRESOURCE,"No pixmap from Xpm data!\n");
				return(FALSE);
			}
		}
#else
		{
			GtError(this,GTEC_MISC,"No Xpm support!\n");
			return(FALSE);
		}
#endif
	}
	if(data)
	{
		if(bm)
		{
			if(depth==1)
				pix=XCreateBitmapFromData(dsp->XDsp(),draw,data,xsize,ysize);
			else
				pix=XCreatePixmapFromBitmapData(dsp->XDsp(),draw,data,xsize,ysize,
						fg,bg,depth);
			if(fname)
			{
				XFree(data);
				data=NULL;
			}
		}
		else
#ifndef GT_NO_XPM_LIB
		{
			attr.valuemask=XpmVisual|XpmColormap|XpmDepth;
			attr.visual=dsp->XDefaultVisual();
			if(cm)
				attr.colormap=cm->XCm();
			else
				attr.colormap=dsp->XDefaultColormap();
			attr.depth=depth;
			if(bgset)
			{
				csym.name="background";
				csym.value=NULL;
				csym.pixel=bg;
				attr.valuemask|=XpmColorSymbols;
				attr.colorsymbols=&csym;
				attr.numsymbols=1;
			}
			if(XpmCreatePixmapFromData(dsp->XDsp(),draw,(char **)data,
																							&pix,&mask,&attr)!=XpmSuccess)
			{
				GtError(this,GTEC_NOXRESOURCE,"No pixmap from Xpm data!\n");
				return(FALSE);
			}
			xsize=attr.width;
			ysize=attr.height;
			hotx=attr.x_hotspot;
			hoty=attr.y_hotspot;
			if(fname)
			{
				XpmFree(data);
				data=NULL;
			}
		}
#else
		{
			GtError(this,GTEC_MISC,"No Xpm support!\n");
			return(FALSE);
		}
#endif
	}
	else
		pix=XCreatePixmap(dsp->XDsp(),draw,xsize,ysize,depth);
	if(!pix)
	{
		GtError(this,GTEC_NOXRESOURCE,"No X-Pixmap!\n");
		return(FALSE);
	}
	dr=pix;
	return(TRUE);
}

/************************************************************************\
*
* Set pixmap attributes.
*
\************************************************************************/
sbool GtPixmap::Set(GtTag *tgs)
{
	tgs=tgs;
	return(TRUE);
}

/************************************************************************\
*
* Get pixmap attributes.
*
\************************************************************************/
void GtPixmap::Get(GtTag *tgs)
{
	GtTag *t;
	ulong *ptr;
	
	GtDrawable::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(ulong *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_BitmapData:
				if(depth==1)
					*ptr=(ulong)data;
				break;
			case GT_XpmData:
				*ptr=(ulong)data;
				break;
			case GT_BitmapName:
				if(depth==1)
					*ptr=(ulong)fname;
				break;
			case GT_XpmName:
				*ptr=(ulong)fname;
				break;
			case GT_XSize:
				*ptr=xsize;
				break;
			case GT_YSize:
				*ptr=ysize;
				break;
			case GT_Depth:
				*ptr=depth;
				break;
			case GT_XPixmap:
				*ptr=(ulong)pix;
				break;
			case GT_XPixmapMask:
				*ptr=(ulong)mask;
				break;
			case GT_HotX:
				*ptr=hotx;
				break;
			case GT_HotY:
				*ptr=hoty;
				break;
			case GT_DontFreeX:
				*ptr=nofreex;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtPixmap::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtPixmap")==0)
		return(TRUE);
	else
		return(GtDrawable::IsSubClassOf(cn));
}
